@@ -9,13 +9,9 @@
 /*
  * Try to use an initial size that will cover default cases. We aren't talking
  * about huge files, so why fiddle about with reallocs?
- * I checked /proc/meminfo sizes on 3 different systems: 598, 644, 654
- * 
- * On newer systems, the size is up to around 930 (2.6.27 kernel)
- *   or 1160  (2.6.28 kernel)
  */
-#define MEMINFO_INIT_SIZE   1279
-#define MEMINFO_STEP_SIZE   256
+#define MEMINFO_INIT_SIZE   4096
+#define MEMINFO_STEP_SIZE   4096
 #define MEMINFO_FILE   "/proc/meminfo"
 
     /*
@@ -28,8 +24,9 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
     static int   first = 1;
     ssize_t      bytes_read;
     char        *b;
-    unsigned long memtotal = 0,  memfree = 0, memshared = 0,
-                  buffers = 0,   cached = 0,
+    int          have_memavail = 0;
+    unsigned long memtotal = 0,  memavail = 0, memfree = 0, memshared = 0,
+                  buffers = 0,   cached = 0, sreclaimable = 0,
                   swaptotal = 0, swapfree = 0;
 
     netsnmp_memory_info *mem;
@@ -70,6 +67,7 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
     close(statfd);
     if (bytes_read <= 0) {
         snmp_log_perror(MEMINFO_FILE);
+        buff[0] = '\0';
     } else {
         buff[bytes_read] = '\0';
     }
@@ -84,6 +82,11 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         if (first)
             snmp_log(LOG_ERR, "No MemTotal line in /proc/meminfo\n");
     }
+    b = strstr(buff, "MemAvailable: ");
+    if (b) {
+        have_memavail = 1;
+        sscanf(b, "MemAvailable: %lu", &memavail);
+    }
     b = strstr(buff, "MemFree: ");
     if (b) 
         sscanf(b, "MemFree: %lu", &memfree);
@@ -133,6 +136,9 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         if (first)
             snmp_log(LOG_ERR, "No SwapFree line in /proc/meminfo\n");
     }
+    b = strstr(buff, "SReclaimable: ");
+    if (b)
+        sscanf(b, "SReclaimable: %lu", &sreclaimable);
     first = 0;
 
 
@@ -151,6 +157,18 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         mem->other = -1;
     }
 
+    if (have_memavail) {
+        mem = netsnmp_memory_get_byIdx(NETSNMP_MEM_TYPE_AVAILMEM, 1);
+        if (mem) {
+            if (!mem->descr)
+                mem->descr = strdup("Available memory");
+            mem->units = 1024;
+            mem->size  = memavail;
+            mem->free  = memavail;
+            mem->other = -1;
+        }
+    }
+
     mem = netsnmp_memory_get_byIdx( NETSNMP_MEM_TYPE_VIRTMEM, 1 );
     if (!mem) {
         snmp_log_perror("No Virtual Memory info entry");
@@ -182,7 +200,7 @@ int netsnmp_mem_arch_load( netsnmp_cache *cache, void *magic ) {
         if (!mem->descr)
              mem->descr = strdup("Cached memory");
         mem->units = 1024;
-        mem->size  = cached;
+        mem->size  = cached + sreclaimable;
         mem->free  = 0;     /* Report cached size/used as equal */
         mem->other = -1;
     }
