@@ -1,3 +1,9 @@
+/*
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
+ */
 #ifndef SNMP_TARGET_H
 #define SNMP_TARGET_H
 
@@ -19,6 +25,7 @@ typedef int     (TargetFilterFunction) (struct targetAddrTable_struct *
 netsnmp_session *get_target_sessions(char *taglist, TargetFilterFunction *,
                                      void *filterArg);
 
-config_require(target/snmpTargetAddrEntry target/snmpTargetParamsEntry)
+config_require(target/snmpTargetAddrEntry_data);
+config_require(target/snmpTargetParamsEntry_data);
 
 #endif                          /* SNMP_TARGET_H */
