@@ -18,12 +18,14 @@
 /*
  * Dependency requirements 
  */
-config_arch_require(solaris2, kernel_sunos5)
+#ifdef solaris2
+config_require(kernel_sunos5);
+#endif
 
     /*
      * Directive to include utility module 
      */
-config_require(util_funcs)
+config_require(util_funcs/header_generic);
 
     /*
      * MIB wants CPU_SYSTEM which is sysinfo CPU_KERNEL + CPU_WAIT 
