@@ -25,47 +25,40 @@ SOFTWARE.
 ******************************************************************/
 #include <net-snmp/net-snmp-config.h>
 
-#if HAVE_STDLIB_H
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_STRING_H
+#ifdef HAVE_STRING_H
 #include <string.h>
 #else
 #include <strings.h>
 #endif
 #include <sys/types.h>
-#if HAVE_NETINET_IN_H
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
 #include <stdio.h>
 #include <ctype.h>
-#if TIME_WITH_SYS_TIME
-# ifdef WIN32
-#  include <sys/timeb.h>
-# else
-#  include <sys/time.h>
-# endif
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
 # endif
 #endif
-#if HAVE_SYS_SELECT_H
+#ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-#if HAVE_WINSOCK_H
-#include <winsock.h>
-#endif
-#if HAVE_NETDB_H
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
-#if HAVE_ARPA_INET_H
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
 
@@ -81,7 +74,7 @@ usage(void)
     fprintf(stderr,
             "  -C APPOPTS\t\tSet various application specific behaviours:\n");
     fprintf(stderr, "\t\t\t  q:  don't print results on success\n");
-    fprintf(stderr, "\n  TYPE: one of i, u, t, a, o, s, x, d, b, n\n");
+    fprintf(stderr, "\n  TYPE: one of i, u, t, a, o, s, x, d, b\n");
     fprintf(stderr,
             "\ti: INTEGER, u: unsigned INTEGER, t: TIMETICKS, a: IPADDRESS\n");
     fprintf(stderr,
@@ -134,7 +127,9 @@ main(int argc, char *argv[])
     size_t          name_length;
     int             status;
     int             failures = 0;
-    int             exitval = 0;
+    int             exitval = 1;
+
+    SOCK_STARTUP;
 
     putenv(strdup("POSIXLY_CORRECT=1"));
 
@@ -142,11 +137,14 @@ main(int argc, char *argv[])
      * get the common command line arguments 
      */
     switch (arg = snmp_parse_args(argc, argv, &session, "C:", optProc)) {
-    case -2:
-        exit(0);
-    case -1:
+    case NETSNMP_PARSE_ARGS_ERROR:
+        goto out;
+    case NETSNMP_PARSE_ARGS_SUCCESS_EXIT:
+        exitval = 0;
+        goto out;
+    case NETSNMP_PARSE_ARGS_ERROR_USAGE:
         usage();
-        exit(1);
+        goto out;
     default:
         break;
     }
@@ -154,13 +152,13 @@ main(int argc, char *argv[])
     if (arg >= argc) {
         fprintf(stderr, "Missing object name\n");
         usage();
-        exit(1);
+        goto out;
     }
     if ((argc - arg) > 3*SNMP_MAX_CMDLINE_OIDS) {
         fprintf(stderr, "Too many assignments specified. ");
         fprintf(stderr, "Only %d allowed in one request.\n", SNMP_MAX_CMDLINE_OIDS);
         usage();
-        exit(1);
+        goto out;
     }
 
     /*
@@ -168,14 +166,15 @@ main(int argc, char *argv[])
      */
     for (; arg < argc; arg++) {
         DEBUGMSGTL(("snmp_parse_args", "handling (#%d): %s %s %s\n",
-                    arg,argv[arg], arg+1 < argc ? argv[arg+1] : NULL,
-                    arg+2 < argc ? argv[arg+2] : NULL));
+                    arg, argv[arg], arg+1 < argc ? argv[arg+1] : "",
+                    arg+2 < argc ? argv[arg+2] : ""));
         names[current_name++] = argv[arg++];
         if (arg < argc) {
             switch (*argv[arg]) {
             case '=':
             case 'i':
             case 'u':
+            case '3':
             case 't':
             case 'a':
             case 'o':
@@ -183,6 +182,7 @@ main(int argc, char *argv[])
             case 'x':
             case 'd':
             case 'b':
+            case 'n': /* undocumented */
 #ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
             case 'I':
             case 'U':
@@ -194,22 +194,20 @@ main(int argc, char *argv[])
             default:
                 fprintf(stderr, "%s: Bad object type: %c\n", argv[arg - 1],
                         *argv[arg]);
-                exit(1);
+                goto out;
             }
         } else {
             fprintf(stderr, "%s: Needs type and value\n", argv[arg - 1]);
-            exit(1);
+            goto out;
         }
         if (arg < argc)
             values[current_value++] = argv[arg];
         else {
             fprintf(stderr, "%s: Needs value\n", argv[arg - 2]);
-            exit(1);
+            goto out;
         }
     }
 
-    SOCK_STARTUP;
-
     /*
      * open an SNMP session 
      */
@@ -219,8 +217,7 @@ main(int argc, char *argv[])
          * diagnose snmp_open errors with the input netsnmp_session pointer 
          */
         snmp_sess_perror("snmpset", &session);
-        SOCK_CLEANUP;
-        exit(1);
+        goto out;
     }
 
     /*
@@ -240,11 +237,10 @@ main(int argc, char *argv[])
         }
     }
 
-    if (failures) {
-        snmp_close(ss);
-        SOCK_CLEANUP;
-        exit(1);
-    }
+    if (failures)
+        goto close_session;
+
+    exitval = 0;
 
     /*
      * do the request 
@@ -282,7 +278,12 @@ main(int argc, char *argv[])
 
     if (response)
         snmp_free_pdu(response);
+
+close_session:
     snmp_close(ss);
+
+out:
+    netsnmp_cleanup_session(&session);
     SOCK_CLEANUP;
     return exitval;
 }
