@@ -17,59 +17,50 @@
  * distributed with the Net-SNMP package.
  */
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/library/openssl_config.h>
+#include <net-snmp/net-snmp-includes.h>
+#include <net-snmp/library/snmp_openssl.h>
+#if defined(HAVE_OPENSSL_DH_H) && defined(HAVE_LIBCRYPTO)
+#include <openssl/dh.h>
+#endif /* HAVE_OPENSSL_DH_H && HAVE_LIBCRYPTO */
 
-#if HAVE_STDLIB_H
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_STRING_H
+#ifdef HAVE_STRING_H
 #include <string.h>
 #else
 #include <strings.h>
 #endif
 #include <sys/types.h>
-#if HAVE_NETINET_IN_H
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
 #include <stdio.h>
 #include <ctype.h>
-#if TIME_WITH_SYS_TIME
-# ifdef WIN32
-#  include <sys/timeb.h>
-# else
-#  include <sys/time.h>
-# endif
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
 # endif
 #endif
-#if HAVE_SYS_SELECT_H
+#ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-#if HAVE_WINSOCK_H
-#include <winsock.h>
-#endif
-#if HAVE_NETDB_H
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
-#if HAVE_ARPA_INET_H
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
 
-#if defined(HAVE_OPENSSL_DH_H) && defined(HAVE_LIBCRYPTO)
-#include <openssl/dh.h>
-#endif /* HAVE_OPENSSL_DH_H && HAVE_LIBCRYPTO */
-
-#include <net-snmp/net-snmp-includes.h>
-
-int             main(int, char **);
-
 #define CMD_PASSWD_NAME    "passwd"
 #define CMD_PASSWD         1
 #define CMD_CREATE_NAME    "create"
@@ -129,6 +120,7 @@ int             doauthkey = 0, doprivkey = 0, uselocalizedkey = 0;
 size_t          usmUserEngineIDLen = 0;
 u_char         *usmUserEngineID = NULL;
 char           *usmUserPublic_val = NULL;
+int             docreateandwait = 0;
 
 
 void
@@ -139,22 +131,24 @@ usage(void)
     fprintf(stderr, " COMMAND\n\n");
     snmp_parse_args_descriptions(stderr);
     fprintf(stderr, "\nsnmpusm commands:\n");
-    fprintf(stderr, "  [options] create     USER [CLONEFROM-USER]\n");
-    fprintf(stderr, "  [options] delete     USER\n");
-    fprintf(stderr, "  [options] cloneFrom  USER CLONEFROM-USER\n");
-    fprintf(stderr, "  [options] activate   USER\n");
-    fprintf(stderr, "  [options] deactivate USER\n");
-    fprintf(stderr, "  [options] [-Ca] [-Cx] changekey [USER]\n");
+    fprintf(stderr, "  [options]               create     USER [CLONEFROM-USER]\n");
+    fprintf(stderr, "  [options]               delete     USER\n");
+    fprintf(stderr, "  [options]               activate   USER\n");
+    fprintf(stderr, "  [options]               deactivate USER\n");
+    fprintf(stderr, "  [options] [-Cw]         cloneFrom  USER CLONEFROM-USER\n");
+    fprintf(stderr, "  [options] [-Ca] [-Cx]   changekey  [USER]\n");
     fprintf(stderr,
-            "  [options] [-Ca] [-Cx] passwd OLD-PASSPHRASE NEW-PASSPHRASE [USER]\n");
+            "  [options] [-Ca] [-Cx]   passwd     OLD-PASSPHRASE NEW-PASSPHRASE [USER]\n");
     fprintf(stderr,
-            "  [options] (-Ca|-Cx) -Ck passwd OLD-KEY-OR-PASSPHRASE NEW-KEY-OR-PASSPHRASE [USER]\n");
+            "  [options] (-Ca|-Cx) -Ck passwd     OLD-KEY-OR-PASS NEW-KEY-OR-PASS [USER]\n");
     fprintf(stderr, "\nsnmpusm options:\n");
     fprintf(stderr, "\t-CE ENGINE-ID\tSet usmUserEngineID (e.g. 800000020109840301).\n");
     fprintf(stderr, "\t-Cp STRING\tSet usmUserPublic value to STRING.\n");
+    fprintf(stderr, "\t-Cw\t\tCreate the user with createAndWait.\n");
+    fprintf(stderr, "\t\t\t(it won't be active until you active it)\n");
     fprintf(stderr, "\t-Cx\t\tChange the privacy key.\n");
     fprintf(stderr, "\t-Ca\t\tChange the authentication key.\n");
-    fprintf(stderr, "\t-Ck\t\tAllows to use localized key (must start with 0x)\n");
+    fprintf(stderr, "\t-Ck\t\tAllows one to use localized key (must start with 0x)\n");
     fprintf(stderr, "\t\t\tinstead of passphrase.\n");
 }
 
@@ -194,10 +188,10 @@ get_USM_DH_key(netsnmp_variable_list *vars, netsnmp_variable_list *dhvar,
                oid *keyoid, size_t keyoid_len) {
     u_char *dhkeychange;
     DH *dh;
+    const BIGNUM *p, *g, *pub_key;
     BIGNUM *other_pub;
     u_char *key;
     size_t key_len;
-    unsigned char *cp;
             
     dhkeychange = (u_char *) malloc(2 * vars->val_len * sizeof(char));
     if (!dhkeychange)
@@ -205,29 +199,33 @@ get_USM_DH_key(netsnmp_variable_list *vars, netsnmp_variable_list *dhvar,
     
     memcpy(dhkeychange, vars->val.string, vars->val_len);
 
-    cp = dhvar->val.string;
-    dh = d2i_DHparams(NULL, (const unsigned char **) &cp,
-                      dhvar->val_len);
+    {
+        const unsigned char *cp = dhvar->val.string;
+        dh = d2i_DHparams(NULL, &cp, dhvar->val_len);
+    }
 
-    if (!dh || !dh->g || !dh->p) {
+    if (dh)
+        DH_get0_pqg(dh, &p, NULL, &g);
+
+    if (!dh || !g || !p) {
         SNMP_FREE(dhkeychange);
         return SNMPERR_GENERR;
     }
 
-    DH_generate_key(dh);
-    if (!dh->pub_key) {
+    if (!DH_generate_key(dh)) {
         SNMP_FREE(dhkeychange);
         return SNMPERR_GENERR;
     }
-            
-    if (vars->val_len != BN_num_bytes(dh->pub_key)) {
+
+    DH_get0_key(dh, &pub_key, NULL);
+    if (vars->val_len != (unsigned int)BN_num_bytes(pub_key)) {
         SNMP_FREE(dhkeychange);
         fprintf(stderr,"incorrect diffie-helman lengths (%lu != %d)\n",
-                (unsigned long)vars->val_len, BN_num_bytes(dh->pub_key));
+                (unsigned long)vars->val_len, BN_num_bytes(pub_key));
         return SNMPERR_GENERR;
     }
 
-    BN_bn2bin(dh->pub_key, dhkeychange + vars->val_len);
+    BN_bn2bin(pub_key, dhkeychange + vars->val_len);
 
     key_len = DH_size(dh);
     if (!key_len) {
@@ -253,7 +251,7 @@ get_USM_DH_key(netsnmp_variable_list *vars, netsnmp_variable_list *dhvar,
 
         printf("new %s key: 0x", keyname);
         for(kp = key + key_len - outkey_len;
-            kp - key < key_len;  kp++) {
+            kp - key < (int)key_len;  kp++) {
             printf("%02x", (unsigned char) *kp);
         }
         printf("\n");
@@ -300,8 +298,12 @@ optProc(int argc, char *const *argv, int opt)
                 optind++;
                 break;
 
+            case 'w':
+                docreateandwait = 1;
+                break;
+
 	    case 'E': {
-	        size_t ebuf_len = 32; /* XXX: MAX_ENGINEID_LENGTH */
+	        size_t ebuf_len = MAX_ENGINEID_LENGTH;
                 u_char *ebuf;
                 if (optind < argc) {
                     if (argv[optind]) {
@@ -352,7 +354,7 @@ main(int argc, char *argv[])
     size_t          name_length = USM_OID_LEN;
     size_t          name_length2 = USM_OID_LEN;
     int             status;
-    int             exitval = 0;
+    int             exitval = 1;
     int             rval;
     int             command = 0;
     long            longvar;
@@ -375,6 +377,8 @@ main(int argc, char *argv[])
         newkul[SNMP_MAXBUF_SMALL], keychange[SNMP_MAXBUF_SMALL],
         keychangepriv[SNMP_MAXBUF_SMALL];
 
+    SOCK_STARTUP;
+
     authKeyChange = authKeyOid;
     privKeyChange = privKeyOid;
 
@@ -382,11 +386,14 @@ main(int argc, char *argv[])
      * get the common command line arguments 
      */
     switch (arg = snmp_parse_args(argc, argv, &session, "C:", optProc)) {
-    case -2:
-        exit(0);
-    case -1:
+    case NETSNMP_PARSE_ARGS_ERROR:
+        goto out;
+    case NETSNMP_PARSE_ARGS_SUCCESS_EXIT:
+        exitval = 0;
+        goto out;
+    case NETSNMP_PARSE_ARGS_ERROR_USAGE:
         usage();
-        exit(1);
+        goto out;
     default:
         break;
     }
@@ -394,11 +401,9 @@ main(int argc, char *argv[])
     if (arg >= argc) {
         fprintf(stderr, "Please specify an operation to perform.\n");
         usage();
-        exit(1);
+        goto out;
     }
 
-    SOCK_STARTUP;
-
     /*
      * open an SNMP session 
      */
@@ -412,7 +417,7 @@ main(int argc, char *argv[])
          * diagnose snmp_open errors with the input netsnmp_session pointer 
          */
         snmp_sess_perror("snmpusm", &session);
-        exit(1);
+        goto out;
     }
 
     /*
@@ -430,7 +435,7 @@ main(int argc, char *argv[])
     pdu = snmp_pdu_create(SNMP_MSG_SET);
     if (!pdu) {
         fprintf(stderr, "Failed to create request\n");
-        exit(1);
+        goto close_session;
     }
 
 
@@ -456,21 +461,25 @@ main(int argc, char *argv[])
             fprintf(stderr,
                     "New passphrase must be greater than %d characters in length.\n",
                     USM_LENGTH_P_MIN);
-            exit(1);
+            goto close_session;
         }
 
         if (oldpass == NULL || strlen(oldpass) < USM_LENGTH_P_MIN) {
             fprintf(stderr,
                     "Old passphrase must be greater than %d characters in length.\n",
                     USM_LENGTH_P_MIN);
-            exit(1);
+            goto close_session;
         }
 
+        DEBUGMSGTL(("9:usm:passwd", "oldpass len %" NETSNMP_PRIz "d, newpass len %" NETSNMP_PRIz "d\n",
+                    strlen(oldpass), strlen(newpass)));
+
         /* 
          * Change the user supplied on command line.
          */
         if ((passwd_user != NULL) && (strlen(passwd_user) > 0)) {
-            session.securityName = passwd_user;
+            SNMP_FREE(session.securityName);
+            session.securityName = strdup(passwd_user);
         } else {
             /*
              * Use own key object if no user was supplied.
@@ -532,7 +541,7 @@ main(int argc, char *argv[])
 	    if (!snmp_hex_to_binary((u_char **) (&buf), &buf_len, &oldkul_len, 0, oldpass)) {
 	      snmp_perror(argv[0]);
 	      fprintf(stderr, "generating the old Kul from localized key failed\n");
-	      exit(1);
+	      goto close_session;
 	    }
 	    
 	    memcpy(oldkul, buf, oldkul_len);
@@ -550,7 +559,7 @@ main(int argc, char *argv[])
 	    if (rval != SNMPERR_SUCCESS) {
 	        snmp_perror(argv[0]);
 	        fprintf(stderr, "generating the old Ku failed\n");
-	        exit(1);
+	        goto close_session;
 	    }
 
 	    /*
@@ -564,8 +573,9 @@ main(int argc, char *argv[])
 	    if (rval != SNMPERR_SUCCESS) {
 	        snmp_perror(argv[0]);
 		fprintf(stderr, "generating the old Kul failed\n");
-		exit(1);
+		goto close_session;
 	    }
+            DEBUGMSGTL(("9:usm:passwd", "oldkul len %" NETSNMP_PRIz "d\n", oldkul_len));
 	}
 	if (uselocalizedkey && (strncmp(newpass, "0x", 2) == 0)) {
 	    /*
@@ -579,7 +589,7 @@ main(int argc, char *argv[])
 	    if (!snmp_hex_to_binary((u_char **) (&buf), &buf_len, &newkul_len, 0, newpass)) {
 	      snmp_perror(argv[0]);
 	      fprintf(stderr, "generating the new Kul from localized key failed\n");
-	      exit(1);
+	      goto close_session;
 	    }
 	    
 	    memcpy(newkul, buf, newkul_len);
@@ -593,7 +603,7 @@ main(int argc, char *argv[])
             if (rval != SNMPERR_SUCCESS) {
                 snmp_perror(argv[0]);
                 fprintf(stderr, "generating the new Ku failed\n");
-                exit(1);
+                goto close_session;
             }
 
 	    rval = generate_kul(session.securityAuthProto,
@@ -604,8 +614,9 @@ main(int argc, char *argv[])
 	    if (rval != SNMPERR_SUCCESS) {
 	        snmp_perror(argv[0]);
 		fprintf(stderr, "generating the new Kul failed\n");
-		exit(1);
+		goto close_session;
 	    }
+            DEBUGMSGTL(("9:usm:passwd", "newkul len %" NETSNMP_PRIz "d\n", newkul_len));
 	}
 
         /*
@@ -614,26 +625,44 @@ main(int argc, char *argv[])
          * they're the same lengths.
          */
         if (doprivkey) {
+            int privtype, properlength;
+            u_char *okp = oldkulpriv, *nkp = newkulpriv;
             if (!session.securityPrivProto) {
                 snmp_log(LOG_ERR, "no encryption type specified, which I need in order to know to change the key\n");
-                exit(1);
+                goto close_session;
             }
-                
-#ifndef NETSNMP_DISABLE_DES
-            if (ISTRANSFORM(session.securityPrivProto, DESPriv)) {
-                /* DES uses a 128 bit key, 64 bits of which is a salt */
-                oldkulpriv_len = newkulpriv_len = 16;
-            }
-#endif
-#ifdef HAVE_AES
-            if (ISTRANSFORM(session.securityPrivProto, AESPriv)) {
-                oldkulpriv_len = newkulpriv_len = 16;
-            }
-#endif
+
+            privtype = sc_get_privtype(session.securityPrivProto,
+                                       session.securityPrivProtoLen);
+            properlength = sc_get_proper_priv_length_bytype(privtype);
+            if (USM_CREATE_USER_PRIV_DES == privtype)
+                properlength *= 2; /* ?? we store salt with key */
+            DEBUGMSGTL(("9:usm:passwd", "proper len %d\n", properlength));
+            oldkulpriv_len = oldkul_len;
+            newkulpriv_len = newkul_len;
             memcpy(oldkulpriv, oldkul, oldkulpriv_len);
             memcpy(newkulpriv, newkul, newkulpriv_len);
+
+            if (oldkulpriv_len > properlength) {
+                oldkulpriv_len = newkulpriv_len = properlength;
+            }
+            else if (oldkulpriv_len < properlength) {
+                rval = netsnmp_extend_kul(properlength,
+                                          session.securityAuthProto,
+                                          session.securityAuthProtoLen,
+                                          privtype,
+                                          usmUserEngineID, usmUserEngineIDLen,
+                                          &okp, &oldkulpriv_len,
+                                          sizeof(oldkulpriv));
+                rval = netsnmp_extend_kul(properlength,
+                                          session.securityAuthProto,
+                                          session.securityAuthProtoLen,
+                                          privtype,
+                                          usmUserEngineID, usmUserEngineIDLen,
+                                          &nkp, &newkulpriv_len,
+                                          sizeof(newkulpriv));
+            }
         }
-            
 
         /*
          * create the keychange string 
@@ -649,24 +678,28 @@ main(int argc, char *argv[])
 	    snmp_perror(argv[0]);
             fprintf(stderr, "encoding the keychange failed\n");
             usage();
-            exit(1);
+            goto close_session;
 	  }
 	}
 
         /* which is slightly different for encryption if lengths are
            different */
 	if (doprivkey) {
+            DEBUGMSGTL(("9:usm:passwd:encode", "proper len %" NETSNMP_PRIz "d, old_len %" NETSNMP_PRIz "d, new_len %" NETSNMP_PRIz "d\n",
+                        oldkulpriv_len, oldkulpriv_len, newkulpriv_len));
 	  rval = encode_keychange(session.securityAuthProto,
                                 session.securityAuthProtoLen,
                                 oldkulpriv, oldkulpriv_len,
                                 newkulpriv, newkulpriv_len,
                                 keychangepriv, &keychangepriv_len);
 
+          DEBUGMSGTL(("9:usm:passwd:encode", "keychange len %" NETSNMP_PRIz "d\n",
+                      keychangepriv_len));
 	  if (rval != SNMPERR_SUCCESS) {
             snmp_perror(argv[0]);
             fprintf(stderr, "encoding the keychange failed\n");
             usage();
-            exit(1);
+            goto close_session;
 	  }
 	}
 
@@ -698,7 +731,7 @@ main(int argc, char *argv[])
         if (++arg >= argc) {
             fprintf(stderr, "You must specify the user name to create\n");
             usage();
-            exit(1);
+            goto close_session;
         }
 
         command = CMD_CREATE;
@@ -710,7 +743,11 @@ main(int argc, char *argv[])
              */
             setup_oid(usmUserStatus, &name_length,
                       usmUserEngineID, usmUserEngineIDLen, argv[arg-1]);
-            longvar = RS_CREATEANDGO;
+            if (docreateandwait) {
+                longvar = RS_CREATEANDWAIT;
+            } else {
+                longvar = RS_CREATEANDGO;
+            }
             snmp_pdu_add_variable(pdu, usmUserStatus, name_length,
                                   ASN_INTEGER, (u_char *) & longvar,
                                   sizeof(longvar));
@@ -749,7 +786,7 @@ main(int argc, char *argv[])
             fprintf(stderr,
                     "You must specify the user name to operate on\n");
             usage();
-            exit(1);
+            goto close_session;
         }
 
         command = CMD_CLONEFROM;
@@ -767,7 +804,7 @@ main(int argc, char *argv[])
             fprintf(stderr,
                     "You must specify the user name to clone from\n");
             usage();
-            exit(1);
+            goto close_session;
         }
 
         setup_oid(usmUserSecurityName, &name_length2,
@@ -785,7 +822,7 @@ main(int argc, char *argv[])
          */
         if (++arg >= argc) {
             fprintf(stderr, "You must specify the user name to delete\n");
-            exit(1);
+            goto close_session;
         }
 
         command = CMD_DELETE;
@@ -803,7 +840,7 @@ main(int argc, char *argv[])
          */
         if (++arg >= argc) {
             fprintf(stderr, "You must specify the user name to activate\n");
-            exit(1);
+            goto close_session;
         }
 
         command = CMD_ACTIVATE;
@@ -821,7 +858,7 @@ main(int argc, char *argv[])
          */
         if (++arg >= argc) {
             fprintf(stderr, "You must specify the user name to deactivate\n");
-            exit(1);
+            goto close_session;
         }
 
         command = CMD_DEACTIVATE;
@@ -876,7 +913,7 @@ main(int argc, char *argv[])
         dhpdu = snmp_pdu_create(SNMP_MSG_GET);
         if (!dhpdu) {
             fprintf(stderr, "Failed to create DH request\n");
-            exit(1);
+            goto close_session;
         }
 
         /* get the current DH parameters */
@@ -929,18 +966,12 @@ main(int argc, char *argv[])
             vars = vars->next_variable;
         }
         if (doprivkey) {
-	    size_t dhprivKeyLen = 0;
-#ifndef NETSNMP_DISABLE_DES
-	    if (ISTRANSFORM(ss->securityPrivProto, DESPriv)) {
-                /* DES uses a 128 bit key, 64 bits of which is a salt */
-	        dhprivKeyLen = 16;
-	    }
-#endif
-#ifdef HAVE_AES
-	    if (ISTRANSFORM(ss->securityPrivProto, AESPriv)) {
-	        dhprivKeyLen = 16;
-	    }
-#endif
+            size_t dhprivKeyLen = 0;
+            int privtype = sc_get_privtype(ss->securityPrivProto,
+                                           ss->securityPrivProtoLen);
+            dhprivKeyLen = sc_get_proper_priv_length_bytype(privtype);
+            if (USM_CREATE_USER_PRIV_DES == privtype)
+                dhprivKeyLen *= 2; /* ?? we store salt with key */
             if (get_USM_DH_key(vars, dhvar,
                                dhprivKeyLen,
                                pdu, "priv",
@@ -954,7 +985,7 @@ main(int argc, char *argv[])
     } else {
         fprintf(stderr, "Unknown command\n");
         usage();
-        exit(1);
+        goto close_session;
     }
 
     /*
@@ -1006,12 +1037,18 @@ main(int argc, char *argv[])
         exitval = 1;
     }
 
+    exitval = 0;
 #if defined(HAVE_OPENSSL_DH_H) && defined(HAVE_LIBCRYPTO)
   begone:
 #endif /* HAVE_OPENSSL_DH_H && HAVE_LIBCRYPTO */
     if (response)
         snmp_free_pdu(response);
+
+close_session:
     snmp_close(ss);
+
+out:
+    netsnmp_cleanup_session(&session);
     SOCK_CLEANUP;
     return exitval;
 }
